{+++ Unit ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Name         :  KeyBar                                                   +
 +  Autor        :  Ing. Thomas Zakostelsky                                  +
 +  EMail        :  Zako@afried.mhs.compuserve.com                           +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  Eine Statusbar die die Funktionstasten F1 bis F10 anzeigt.               +
 +  Wenn die Shift, Ctrl oder Alt-Taste gedrckt wird, dann wird automatisch +
 +  der entsprechende Text angezeigt. Dieser kann ber die Properties zur    +
 +  Designzeit erstellt werden. Desweiteren wird ein Event ausgelst, wenn   +
 +  die linke Mousetaste bettigt wird. Dabei wird das Panel in dem dieses   +
 +  Ereigniss stattfindet auf die var FUNKTION_KEY umgelegt.                 +
 +---------------------------------------------------------------------------+
 +  Anwendung :                                                              +
 +  1.) Die Komponenten auf dem Formular plazieren.                          +
 +  2.) Ein 'OnKeyDown' und 'OnKeyUp'-Event fr das Formular erzeugen.       +
 +  3.) Im Event dem KeyState der Komponente die var SHIFT zuweisen.         +
 +  4.) Ein 'OnMouseDown'-Event fr die Komponente erzeugen.                 +
 +  5.) Dort knnen ber die Anweisung                                       +
 +         Case (FUNKTION_KEY + SPECIAL_KEY) of                              +
 +           1 .. 10 : Funktionen fr       F1..F10                          +
 +          11 .. 20 : Funktionen fr Shift F1..F10                          +
 +          21 .. 30 : Funktionen fr Ctrl  F1..F10                          +
 +          31 .. 40 : Funktionen fr Alt   F1..F10                          +
 +      die Ereignisse entsprechen ausgelst werden.                         +
 +---------------------------------------------------------------------------+
 +  Interface    :                                                           +
 +     TKeyBar = class(TStatusBar)                                           +
 +       Constructor Create (AOwner : TComponent) ; Override;                +
 +       Procedure   WriteNormal   (Value : TStringList);                    +
 +       Procedure   WriteShift    (Value : TStringList);                    +
 +       Procedure   WriteCtrl     (Value : TStringList);                    +
 +       Procedure   WriteAlt      (Value : TStringList);                    +
 +       Procedure   WriteKeyState (Value : TShiftState);                    +
 +       Property    Funktion_Key  : Byte                                    +
 +       Property    Special_Key   : Byte                                    +
 +       Property    Keys_Normal   : TStringList                             +
 +       Property    Keys_Shift    : TStringList                             +
 +       Property    Keys_Ctrl     : TStringList                             +
 +       Property    Keys_Alt      : TStringList                             +
 +       Property    KeyState      : TShiftState                             +
 +       property    OnResize      : TNotifyEvent                            +
 +   procedure Register;                                                     +
 +---------------------------------------------------------------------------+
 +  History :                                                                +
 +  V 1.0     03.09.98    Erstausgabe                                        +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
unit KeyBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;

type
  TKeyBar = class(TStatusBar)
  private
    { Private-Deklarationen }
    FstrlNormal   : TStringList;        { Liste fr die Funktionstasten ohne Sondertaste    }
    FstrlShift    : TStringList;        { Liste fr die Funktionstasten mit <SHIFT> - Taste }
    FstrlCtrl     : TStringList;        { Liste fr die Funktionstasten mit <CTRL> - Taste  }
    FstrlAlt      : TStringList;        { Liste fr die Funktionstasten mit <ALT> - Taste   }
    FKeyState     : TShiftState;        { Status der Sondertasten                           }
    FFunktion_Key : Byte;               { Welche Funktionstaste (1..10) wurde gedrckt      }
    FSpecial_Key  : Byte;               { Welche Sondertaste                                }

    FOnResize     : TNotifyEvent;
    FOnMouseDown  : TMouseEvent;

    Procedure DoMouseDown   (var Message: TWMMouse; Button: TMouseButton;
                             Shift: TShiftState);
    Procedure WMLButtonDown (var Message: TWMLButtonDown); Message WM_LBUTTONDOWN;
    Procedure WMSize        (var Message: TWMSize);        Message WM_SIZE;

  protected
    procedure MouseDown( Button: TMouseButton; Shift: TShiftState;
                         X, Y : Integer); dynamic;
  public
    { Public-Deklarationen }
    Constructor Create        (AOwner : TComponent) ; Override;
    Procedure   WriteNormal   (Value : TStringList);
    Procedure   WriteShift    (Value : TStringList);
    Procedure   WriteCtrl     (Value : TStringList);
    Procedure   WriteAlt      (Value : TStringList);
    Procedure   WriteKeyState (Value : TShiftState);

  published
    { Published-Deklarationen }
    Property    Funktion_Key  : Byte         Read FFunktion_Key;    { Read only }
    Property    Special_Key   : Byte         Read FSpecial_Key;     { Read only }

    Property    Keys_Normal  : TStringList  Read FstrlNormal  Write WriteNormal;
    Property    Keys_Shift   : TStringList  Read FstrlShift   Write WriteShift;
    Property    Keys_Ctrl    : TStringList  Read FstrlCtrl    Write WriteCtrl;
    Property    Keys_Alt     : TStringList  Read FstrlAlt     Write WriteAlt;
    Property    KeyState     : TShiftState  Read FKeyState    Write WriteKeyState;

    Property    OnResize     : TNotifyEvent Read FOnResize    Write FOnResize;
  end;

procedure Register;

implementation

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  Register                                                 +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung : Registriert die Komponente im Register TOOLS.             +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
procedure Register;
begin
  RegisterComponents('TOOLS', [TKeyBar]);
end;

{+++ Constructor +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.Create                                           +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  Erzeugt die Komponente, die Stringlisten und 10 Panels fr die Funk-     +
 +  tionstasten. Die Listen werden mit dem Text 'Fx' vorbelegt. Die Breite   +
 +  wird auf 250 festgelegt und die einzelnen Panels mit 50. Der KeyState    +
 +  wird auf [] gesetzt.                                                     +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +   AOwner : TComponent               Eigentmer der Komponente             +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Constructor TKeyBar.Create(AOwner : TComponent);
Var x : Byte;
Begin
  inherited Create(AOwner);
  Height :=  20;
  Width  := 250;
  FstrlNormal  := TStringList.Create;
  FstrlShift   := TStringList.Create;
  FstrlCtrl    := TStringList.Create;
  FstrlAlt     := TStringList.Create;
  For  x := 1 to 10 do
  Begin
    FstrlNormal.Add('F'+IntToStr(x));
    FstrlShift.Add ('F'+IntToStr(x));
    FstrlCtrl.Add  ('F'+IntToStr(x));
    FstrlAlt.Add   ('F'+IntToStr(x));
    Panels.Add;
    Panels.Items[x-1].Text      := 'F' + IntToStr(x) + ' ';
    Panels.Items[x-1].Width     := 25;
    Panels.Items[x-1].Style     := psText;
    Panels.Items[x-1].Bevel     := pbLowered;
    Panels.Items[x-1].Alignment := taLeftJustify;
  End;
  FKeyState   := [];
  WriteKeyState([]);
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.WMSize                                           +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  Wird bei einer Grennderung der Komponente aufgerufen. Die einzelnen   +
 +  Panels werden dabei so skaliert, da sie gleich gro sind. Wenn die      +
 +  Breite dabei kleiner als 50 wre, dann wird sie auf 50 gesetzt.          +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +   var Message: TWMSize              Windows-Message                       +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
procedure TKeyBar.WMSize(var Message: TWMSize);
Var x,wSize : Word;
begin
  if Assigned(FOnResize) then FOnResize(Self);
  if (GetParentForm(self).Width > 500) Then wSize := GetParentForm(self).Width div 10
                                       Else wSize := 50;
  For x := 0 to 9 do panels.items[x].width := wSize;
end;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.MouseDown                                        +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  Ermittelt das Panel, welches ausgewhlt wurde und ruft dann das Original-+
 +  ereigniss auf.                                                           +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +   Button :  TMouseButton            Status der Maustasten                 +
 +   Shift  : TShiftState              Status der Spezialtasten              +
 +   X,y    : Integer                  Position des Mauszeigers              +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
procedure TKeyBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y : Integer);
begin
  FFunktion_Key := (X div Panels[0].Width) + 1;
  if Assigned( FOnMouseDown ) then FOnMouseDown( Self, Button, Shift, X, Y );
end;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.DoMouseDown                                      +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  bersetzt die Windowsbotschaft.                                          +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +   var Message : TWMMouse            Windows-Message                       +
 +   Button :  TMouseButton            Status der Maustasten                 +
 +   Shift  : TShiftState              Status der Spezialtasten              +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.DoMouseDown(Var Message : TWMMouse; Button : TMouseButton;
                              Shift : TShiftState);
Begin
  with Message do MouseDown(Button, Shift, XPos, YPos);
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.DoMouseDown                                      +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  Wird von Windows ausgelst, wenn die linke Maustaste gedrckt wird.      +
 +  Dabei wird zuerst die eigenen Routinen aufgerufen und dann das Original. +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +   var Message : TWMLButtonDown      Windows-Message                       +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.WMLButtonDown(Var Message : TWMLButtonDown);
Begin
  DoMouseDown(Message, mbLeft, []);
  inherited;
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.WriteNormal                                      +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  bernimmt die Stringliste fr die Funktionstasten im Normalzustand.      +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.WriteNormal;
Begin
  FstrlNormal.Clear;
  FstrlNormal.Assign(Value);
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.WriteShift                                       +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung : Text                                                      +
 +  bernimmt die Stringliste fr die Funktionstasten mit <SHIFT>.           +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.WriteShift;
Begin
  FstrlShift.Clear;
  FstrlShift.Assign(Value);
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.WriteCtrl                                        +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung : Text                                                      +
 +  bernimmt die Stringliste fr die Funktionstasten mit <CTRL>.            +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.WriteCtrl;
Begin
  FstrlCtrl.Clear;
  FstrlCtrl.Assign(Value);
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.WriteAlt                                         +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung : Text                                                      +
 +  bernimmt die Stringliste fr die Funktionstasten mit <ALT>.             +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.WriteAlt;
Begin
  FstrlAlt.Clear;
  FstrlAlt.Assign(Value);
End;

{+++ Procedure +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 +  Modulname    :  TKeyBar.WriteKeyState                                    +
 +  Datum        :  03.09.1998                                               +
 +---------------------------------------------------------------------------+
 +  Beschreibung :                                                           +
 +  bernimmt den Status der Sondertasten und setzt dann die Texte der       +
 +  Panels.                                                                  +
 +---------------------------------------------------------------------------+
 +  Parameter    :                                                           +
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
Procedure TKeyBar.WriteKeyState;
Var x : Byte;
Begin
  FKeyState := Value;
  FSpecial_Key := 0;
  If (Value = [ssShift]) Then FSpecial_Key := 10;
  If (Value = [ssCtrl])  Then FSpecial_Key := 20;
  If (Value = [ssAlt])   Then FSpecial_Key := 30;
  For x := 0 to 9 do
    Case FSpecial_Key Of
       0 : Panels[x].Text := FstrlNormal.Strings[x];
      10 : Panels[x].Text := FstrlShift.Strings[x];
      20 : Panels[x].Text := FstrlCtrl.Strings[x];
      30 : Panels[x].Text := FstrlAlt.Strings[x];
    End;
End;

end.
